# -*- coding: utf-8 -*-
"""
Created on Thu Nov  3 11:58:45 2022
@author: smvanessen
"""

# Clear all variables
from IPython import get_ipython
get_ipython().magic('reset -sf')

# Standard toolboxes
import numpy as np
import matplotlib.pyplot as plt
import sys
import os

# Allow for subsampling of large datasets for plotting & saving as PDF
sys.path.append('functions')
from functions.subsample_routines import subsample_binned_2D
from functions.dedicated_screening_functions import calc_SQI_orig

# Close all figures
plt.close('all')


#############
# START INPUT
#############

# HF symbol and unit
HFsymb = 'C \prime\prime'
HFunit = 'm'

# Indicator cases
indcases = ['GoodInd','WorseInd']; indaliases = ['C_{good}\prime','C_{worse}\prime']
   
# Plot orientation (only used when there are 2 cases defined above)
orient = 'hor'
# orient = 'ver'

# Folder name where input files are stored (files to read)
inputfolder = 'inputfiles_waves'
    
# Folder name in which to store files and figures (rel. to present folder)
figfolder = 'out_figs_paper'

# Number of seeds (need to comply with available data)
Ns = 50 

# Maximum plot limit for HF input [m]
maxvalinHF = 15

#############
# END INPUT
#############

# Initialise
colgrey = [0.7,0.7,0.7]
if len(indcases) == 4:
    fig7 = plt.figure(7,figsize=(4.8,3.75))
    fig8 = plt.figure(8,figsize=(4.8,3.75))
elif len(indcases) == 2 and orient == 'hor':
    # fig7 = plt.figure(7,figsize=(4.8,1.8))
    fig7 = plt.figure(7,figsize=(4,1.8))
    fig8 = plt.figure(8,figsize=(4.8,1.8))
elif len(indcases) == 2 and orient == 'ver':
    fig7 = plt.figure(7,figsize=(2.4,3.75))
    fig8 = plt.figure(8,figsize=(2.4,3.75))
xticksnew = np.arange(0,16,5)
yticksnew = np.arange(0,16,5)

for nn,indcase in enumerate(indcases):
    
    indalias = indaliases[nn]
    
    # Check wether output directories exist, if not make them
    isExist = os.path.exists(figfolder)
    if not isExist:
       os.makedirs(figfolder)
    
    # Read true MCS time trace data from CSV
    AllTrueDat_scr = np.loadtxt(f'{inputfolder}/Lin2nd_trueLFandHFdata_{indcase}_{Ns}seeds_scr.csv', delimiter=',')
    t_peaks_lin_all = AllTrueDat_scr[:,0]
    peaks_lin_all = AllTrueDat_scr[:,1]
    t_peaks_2nd_all = AllTrueDat_scr[:,2]
    peaks_2nd_all = AllTrueDat_scr[:,3]
        
    # Subsample for the scatter plots
    maxval = 15
    xbinsize = maxval/50
    ybinsize = maxval/50
    xbins = np.arange(0,maxval,xbinsize)
    ybins = np.arange(0,maxval,ybinsize)
    peaks_lin_all_samp,peaks_2nd_all_samp = subsample_binned_2D(peaks_lin_all,peaks_2nd_all,xbins,ybins)
    
    # Scatter plot indicator vs NL response peaks and selected samples
    plt.figure(7)
    if len(indcases) == 4:
        plt.subplot(2,2,nn+1)
    elif len(indcases) == 2 and orient == 'hor':
        plt.subplot(1,2,nn+1)
    elif len(indcases) == 2 and orient == 'ver':
        plt.subplot(2,1,nn+1)
    plt.plot(peaks_lin_all_samp,peaks_2nd_all_samp,marker='o',color=colgrey,label='All peaks',linestyle = 'None')
    plt.plot(np.array([0,15]),np.array([0,15]),'--',color='grey')
    if len(indcases) == 4:
        if nn > 1:
            # plt.xlabel('LF indicator: linear wave crests [m]')
            plt.xticks(xticksnew)
        else:
            plt.xticks(xticksnew,labels=[])
        if nn == 0 or nn == 2:
            # plt.ylabel('HF response: 2$^{nd}$-\n order wave crests [m]')
            plt.yticks(yticksnew)
        else:
            plt.yticks(yticksnew,labels=[])
    elif len(indcases) == 2 and orient == 'hor':
        plt.xticks(xticksnew)
        if nn == 0:
            plt.yticks(yticksnew)
        else:
            plt.yticks(yticksnew,labels=[])
    elif len(indcases) == 2 and orient == 'ver':
        plt.yticks(yticksnew) 
        if nn == 1:
            plt.xticks(xticksnew)
        else:
            plt.xticks(xticksnew,labels=[])
         
    plt.xlim([0,15])
    plt.ylim([0,15])
    plt.xlabel(r'$%s$ [m]' % indalias)
    if nn == 0:
        # plt.ylabel('$C \'\'$ [m]')
        plt.ylabel(r'$%s$ [%s]' %(HFsymb,HFunit))
    # plt.title(f'{indalias}')
    
    # Threshold range for SQI calculation
    thr_range = np.linspace(0,maxvalinHF,100)
    
    # Calculate SQI over threshold range
    SQI = calc_SQI_orig(peaks_lin_all,peaks_2nd_all,thr_range)
    
    plt.figure(8)
    if orient == 'hor':
        plt.subplot(1,2,nn+1)
    elif orient == 'ver':
        plt.subplot(2,1,nn+1)
    plt.plot(thr_range,SQI)
    plt.xlabel(r'Threshold $%s$ [%s]' % (HFsymb, HFunit))
    if nn == 0:
        plt.ylabel('SQI [-]')
    plt.title(r'Indicator: $%s$' %(indalias))
    plt.xlim([0,maxvalinHF])
    plt.ylim([0,10])
    plt.yticks([0,2,4,6,8,10])
    
    # Calculate SQI10 (same as in OE paper; using the SQI at the threshold
    # corresponding to the 10th highest screened HF peak)
    HFsorted = np.sort(peaks_2nd_all)
    HF10th = HFsorted[-10]
    SQI10 = np.interp(HF10th,thr_range,SQI)
    
    plt.figure(8)
    plt.plot([HF10th,HF10th],[0,SQI10],'--k')
    plt.text(HF10th*1.02,0.5*SQI10,'10th\nmax. peak')
    plt.plot([0,HF10th],[SQI10,SQI10],'--k')
    plt.text(HF10th*0.1,1.1*SQI10,'SQI10')
    
    print(f'Indicator: {indalias}')
    print(f'10th highest HF peak: {HF10th} {HFunit}')
    print(f'SQI10: {SQI10}')
    
    # Calculate mean SQI over full range (same as in the OE paper)
    SQImean = np.nanmean(SQI)
    print(f'SQImean: {SQImean}')

# # Add shared axes labels and save figure
# if len(indcases) == 4:
#     fig7.text(0.5, 0.02, 'LF indicator: linear wave crests [m]', ha='center')
#     fig7.text(0.03, 0.5, 'HF response: 2$^{nd}$-order wave crests [m]', va='center', rotation='vertical')
# elif len(indcases) == 2 and orient == 'hor':
#     fig7.text(0.5, -0.1, 'LF indicator crests [m]', ha='center')
#     fig7.text(0.03, 0.5, 'HF response crests [m]', va='center', rotation='vertical')
# elif len(indcases) == 2 and orient == 'ver':
#     fig7.text(0.5, 0.02, 'LF indicator crests [m]', ha='center')
#     fig7.text(-0.05, 0.5, 'HF response crests [m]', va='center', rotation='vertical')
plt.figure(7)
plt.savefig(f'{figfolder}/Fig16_Scatters_App1.pdf',bbox_inches='tight')
plt.figure(8)
plt.savefig(f'{figfolder}/FigXX_SQI_App1.pdf',bbox_inches='tight')

