# -*- coding: utf-8 -*-
"""
Created on Thu Nov  3 11:58:45 2022
@author: smvanessen
"""

# Clear all variables
from IPython import get_ipython
get_ipython().magic('reset -sf')

# Standard toolboxes
import numpy as np
import matplotlib.pyplot as plt
import sys
import os

# Required to load Matlab .mat files
import scipy.io


# Allow for subsampling of large datasets for plotting & saving as PDF
sys.path.append('functions')
from functions.subsample_routines import subsample_binned_2D
from functions.dedicated_screening_functions import calc_SQI_orig

# Close all figures
plt.close('all')

#############
# START INPUT
#############

# HF signal name (must match input file names), and input for ticks of plot
HFsigname = 'FX'
HFsymb = 'F_x \prime\prime'
HFunit = 'kN'
maxvalinHF = 15000.1
stepinHF = 5000

# Indicator cases (LFsignames must match input file names), and input for ticks of plot
LFsignames = ['Peaks FXtot (coarse)','Peaks RelSBA']
# indaliases = ['Coarse mesh ComFLOW green water force peaks','Coarse mesh ComFLOW RWE peaks at fore deck']
indaliases = ['F_x \prime','R_{deck} \prime']
indunits = ['kN','m']
maxvalsinLF = [4000.1,6.01]
stepsinLF = [1000,2]

# Tool used to calculate LF indicator (must match input file names)
toolLF = 'MAR3';

# Plot orientation (only used when there are 2 cases defined above)
orient = 'hor'
# orient = 'ver'

# Folder name where input files are stored (files to read)
inputfilepath = 'C:/Users/smvanessen/Documents/PhD/4_GPR_waves_OMAE_RESS_2023-24/python/3_UpdateAfterDraft/inputfiles_gw/' # Few selected

# Testnumber
testno = '10050401'; Tdata = 1819; Tp = 9.7; Vs = 0.5144444*4.6; mu = np.pi
    
# Folder name in which to store files and figures (rel. to present folder)
figfolder = 'out_figs_paper'

#############
# END INPUT
#############

# Initialise
colgrey = [0.7,0.7,0.7]
if orient == 'hor':
    # fig7 = plt.figure(7,figsize=(4.8,1.8))
    fig7 = plt.figure(7,figsize=(4,1.8))
    fig8 = plt.figure(8,figsize=(4.8,1.8))
elif orient == 'ver':
    fig7 = plt.figure(7,figsize=(2.4,3.75))
    fig8 = plt.figure(8,figsize=(2.4,3.75))

for nn,indcase in enumerate(LFsignames):
    
    indalias = indaliases[nn]
    LFsigname = LFsignames[nn]
    indunit = indunits[nn]
    maxvalinLF = maxvalsinLF[nn]
    stepinLF = stepsinLF[nn]
    
    xticksnew = np.arange(0,maxvalinLF,stepinLF)
    yticksnew = np.arange(0,maxvalinHF,stepinHF)
    
    # Check wether output directories exist, if not make them
    isExist = os.path.exists(figfolder)
    if not isExist:
       os.makedirs(figfolder)
    
    ###### Read true MCS time trace data
    
    # Assemble file name of files with matched peaks
    if testno == '100504':
        filen = f'{inputfilepath}{testno}_{LFsigname}_MatchedPeaks_IND_{HFsigname}_oneprobe_inclCFD.mat'
    elif testno == '10050401': 
        filen = f'{inputfilepath}{testno}_{LFsigname}_CFD_MatchedPeaks_IND_{HFsigname}.mat'  
    else:
        filen = f'{inputfilepath}{testno}_{LFsigname}_MatchedPeaks_IND_{HFsigname}_oneprobe.mat'    
    
    # Load mat file with scatter plot data
    mat1 = scipy.io.loadmat(filen)

    # Extract correct data
    t_peaks = mat1[f't_p{toolLF}']
    peaks_lf_all = np.squeeze(mat1[f'p{toolLF}'])
    peaks_hf_matched = np.squeeze(mat1[f'mFXcorr_{toolLF}'])
    
    # # Subsample for the scatter plots (not necesary, few points)
    # xbinsize = 0.1
    # ybinsize = 0.1
    # xbins = np.arange(0,15,xbinsize)
    # ybins = np.arange(0,15,ybinsize)
    # peaks_lf_all_samp,peaks_hf_matched_samp = subsample_binned_2D(peaks_lf_all,peaks_hf_matched,xbins,ybins)
    
    # Also extract 'true' HF distribution
    filen_true = f'{inputfilepath}{testno}_FXpeaks_versus_Impulses.mat'
    mat2 = scipy.io.loadmat(filen_true) 
    # mat2.keys()
    peaks_hf_true = np.squeeze(mat2['pFX'])
    
    
    ###################################
    
    # Scatter plot indicator vs NL response peaks and selected samples
    maxVal = max(np.nanmax(peaks_lf_all),np.nanmax(peaks_hf_matched))
    plt.figure(7)
    if orient == 'hor':
        plt.subplot(1,2,nn+1)
    elif orient == 'ver':
        plt.subplot(2,1,nn+1)
    plt.plot(peaks_lf_all,peaks_hf_matched,marker='o',color=colgrey,label='All peaks',linestyle = 'None')
    # if nn == 0:
    #     plt.plot(np.array([0,maxVal]),np.array([0,maxVal]),'--',color='grey')
    if orient == 'hor':
        plt.xticks(xticksnew)
        if nn == 0:
            plt.yticks(yticksnew)
        else:
            plt.yticks(yticksnew,labels=[])
    elif orient == 'ver':
        plt.yticks(yticksnew) 
        if nn == 1:
            plt.xticks(xticksnew)
        else:
            plt.xticks(xticksnew,labels=[])
    plt.xlim([0,maxvalinLF])
    plt.ylim([0,maxvalinHF])
    plt.xlabel(r'$%s$ [%s]' % (indalias, indunit) )
    if nn == 0:
        # plt.ylabel('$F_x \'\'$ [kN]')
        plt.ylabel(r'$%s$ [%s]' %(HFsymb,HFunit))
    
    
    ###################################
    
    # Threshold range for SQI calculation
    thr_range = np.linspace(0,maxvalinHF,100)
    
    # Calculate SQI over threshold range
    SQI = calc_SQI_orig(peaks_lf_all,peaks_hf_matched,thr_range)
    
    plt.figure(8)
    if orient == 'hor':
        plt.subplot(1,2,nn+1)
    elif orient == 'ver':
        plt.subplot(2,1,nn+1)
    plt.plot(thr_range,SQI)
    plt.xlabel(r'Threshold $%s$ [%s]' % (HFsymb, HFunit))
    if nn == 0:
        plt.ylabel('SQI [-]')
    plt.title(r'Indicator: $%s$' %(indalias))
    plt.xlim([0,maxvalinHF])
    plt.ylim([0,4])
    
    # Calculate SQI10 (same as in OE paper; using the SQI at the threshold
    # corresponding to the 10th highest screened HF peak)
    # HFsorted = np.sort(peaks_hf_matched)
    HFsorted = np.sort(peaks_hf_true)
    HF10th = HFsorted[-10]
    SQI10 = np.interp(HF10th,thr_range,SQI)
    
    plt.figure(8)
    plt.plot([HF10th,HF10th],[0,SQI10],'--k')
    plt.text(HF10th*1.02,0.5*SQI10,'10th\nmax. peak')
    plt.plot([0,HF10th],[SQI10,SQI10],'--k')
    plt.text(HF10th*0.1,1.1*SQI10,'SQI10')
    
    print(f'Indicator: {indalias}')
    print(f'10th highest HF peak: {HF10th} {HFunit}')
    print(f'SQI10: {SQI10}')
    
    # Calculate mean SQI over full range (same as in the OE paper)
    SQImean = np.nanmean(SQI)
    print(f'SQImean: {SQImean}')
    
    
    # # Alternative: mean over 10% highest HF peaks?
    # HF10perc = int(len(HFsorted)/10)
    
    
    
    
    
    
# Add shared axes labels and save figure
# if orient == 'hor':
#     fig7.text(0.5, -0.1, 'LF indicator crests [m]', ha='center')
#     fig7.text(0.03, 0.5, 'HF response crests [m]', va='center', rotation='vertical')
# elif orient == 'ver':
#     fig7.text(0.5, 0.02, 'LF indicator crests [m]', ha='center')
#     fig7.text(-0.05, 0.5, 'HF response crests [m]', va='center', rotation='vertical')
plt.figure(7)
plt.savefig(f'{figfolder}/Fig16_Scatters_App3.pdf',bbox_inches='tight')
plt.figure(8)
plt.savefig(f'{figfolder}/FigXX_SQI_App3.pdf',bbox_inches='tight')

