# -*- coding: utf-8 -*-
"""
Created on Wed Sep  6 16:47:18 2023

@author: smvanessen
"""

# Clear all variables
from IPython import get_ipython
get_ipython().magic('reset -sf')

# Standard toolboxes
import numpy as np
import matplotlib.pyplot as plt
import sys

# Required to load Matlab .mat files
import scipy.io

# Import adaptive screening function
sys.path.append('functions')
from functions.Adaptive_Screening_v8 import adaptive_screening
# from functions.Adaptive_Screening_v7_GPfit_insteadofGPR3 import adaptive_screening as adaptive_screening_gpfit

# Import some other custom functions used to analyse the true validation data and plot results
from functions.distribution_scripts import calc_distribution
from functions.T_enc import calc_T_enc

# Close all figures
plt.close('all')


#############
# START INPUT
#############

# Selected acquisition function and number of considered iterations for GPR
acqHF = 'USMV'
# acqHF = 'MD'
maxitHF = 8

# Selected acquisition function and number of considered iterations for MF-GPR
acqMF = 'USMV'
# acqMF = 'MD'
maxitMF = 8
# maxitMF = 28

# # Generalised Pareto fitting 
# acqGPfit = 'MD'
# maxitGPfit = 8 

# Folder name where input files are stored (files to read)
# filepath = 'O:/Journal article OE 2021/matlab/MatFiles/' # All mat files
filepath = 'C:/Users/smvanessen/Documents/PhD/4_GPR_waves_OMAE_RESS_2023-24/python/3_UpdateAfterDraft/inputfiles_gw/' # Few selected

# Testnumber (corresponding to the CRS GWD test matrix, and the OE 2021 paper)
# Tdata is test duration in [s], Tp is the peak wave period in [s], Vs is the
# vessel forward speed in [m/s] and mu is the heading in [rad] (180 deg = pi rad = head waves)
# testno = '100504'; Tdata = 11635; Tp = 9.7; Vs = 0.5144444*4.6; mu = np.pi
# testno = '100506'; Tdata = 11154; Tp = 12.1; Vs = 0.5144444*4.6; mu = np.pi
# testno = '100505'; Tdata = 11256; Tp = 12.1; Vs = 0.5144444*4.6; mu = np.pi
testno = '10050401'; Tdata = 1819; Tp = 9.7; Vs = 0.5144444*4.6; mu = np.pi

# HF signal name (almost always FX = total force in x-direction on accommodation from the experiments)
# (HFsigname must match file names in path above; HFnewname is the name used in the plots below)
HFsigname = 'FX'; HFnewname = 'FX accommodation'; HFunit = 'kN'; HFsymb = 'fx'

# LF signal name (LFsigname must match file names in path above; LFnewname is the name used in the plots below)
# LFsigname = 'Peaks RelBow'; LFnewname = 'PeaksRelBow'; LFunit = 'm'; LFsymb = 'relbow' # Available for all full tests
# LFsigname = 'Peaks RelSt19'; LFnewname = 'PeaksRelSt19'; LFunit = 'm'; LFsymb = 'relst19' # Available for all full tests
# LFsigname = 'Peaks W11 from W1'; LFnewname = 'PeaksW11'; LFunit = 'm'; LFsymb = 'w11' # Available for all full tests
# LFsigname = 'Peaks FXrow1 (coarse)'; LFnewname = 'PeaksFXrow1Coarse'; LFunit = 'kN'; LFsymb = 'fx1c' # Only available for single run 10050401
LFsigname = 'Peaks FXtot (coarse)'; LFnewname = 'PeaksFXCoarse'; LFunit = 'kN'; LFsymb = 'fxc' # Only available for single run 10050401
# LFsigname = 'Peaks RelSBA'; LFnewname = 'PeaksRelSBA'; LFunit = 'm'; LFsymb = 'rs' # Available for all full tests

# Source of the LF indicator signal (tool to calculate this signal). For a description, see the OE 2021 paper.
# toolLF = 'EXP'; toolname = 'Experimental indicator' # Experimental; available for all tests and the single run 10050401
# toolLF = 'PRE'; toolname = 'Indicator calculated using PRECAL\_R'  # PRECAL_R; available for all tests and the single run 10050401
# toolLF = 'FAT'; toolname = 'Indicator calculated using FATIMA'  # FATIMA; available for all tests and the single run 10050401
# toolLF = 'CFD'; toolname = 'Indicator calculated using OpenFOAM (vA)' # OpenFOAM A of ABS; available for test 100504
# toolLF = 'MAR2'; toolname = 'Indicator calculated using ComFLOW (v2)' # ComFLOW 2 of MARIN; availabe for single run 10050401
toolLF = 'MAR3'; toolname = 'Indicator calculated using ComFLOW (v3)' # ComFLOW 3 of MARIN; availabe for single run 10050401

# Define 'casename' (will be on all output plots and filenames)
casename = f'GW_{LFnewname}_{toolLF}'

# Output folder name in which to store files and figures (rel. to present folder)
filefolder = 'out_files_gw_LP'
figfolder = 'out_figs_gw_LP'

# Target exposure duration [s]
# dur = 1200 # 20 min
# dur = 600 # 10 min
dur = 300 # 5 min

# Initial sample option (see Adaptive Screening script)
init_sampopt = 'A' # subset of LF samples themselves
# init_sampopt = 'B' # GEV fit to LF samples

# Probabilty of exceedance (PoE) range for the prediction
if dur == 600:
    prange_low = 5*10**(-3)
    prange_high = 5*10**(-2)
elif dur == 300:
    prange_low = 1.5*10**(-2) # PAPER!
    prange_high = 8*10**(-2) # PAPER!
elif dur == 1200:
    prange_low = 5*10**(-3)
    prange_high = 2*10**(-2)
    
prange_nostep = 201
prange = np.linspace(prange_low,prange_high,prange_nostep)[:, None]

# Probability of exceedance (PoE) levels of two initial samples
init_plevels = np.array([0.005,0.05])


#############
# END INPUT
#############

# Variable symbols and units (to be used in plots generated by AS scipt)
# Format: ['LF variable symbol','HF variable symbol'], same for unit
varsymb = [LFsymb,HFsymb]
varunit = [LFunit,HFunit]
      
# Derive target exposure duration probability level, accounting for vessel speed and heading
Tpe = calc_T_enc(Tp,Vs,mu)
ne_exp = dur/Tpe
 
# Assemble file name - the 'oneprobe' data seem to the the latest versions
if testno == '100504':
    filen = f'{filepath}{testno}_{LFsigname}_MatchedPeaks_IND_{HFsigname}_oneprobe_inclCFD.mat'
elif testno == '10050401': 
    filen = f'{filepath}{testno}_{LFsigname}_CFD_MatchedPeaks_IND_{HFsigname}.mat'  
else:
    filen = f'{filepath}{testno}_{LFsigname}_MatchedPeaks_IND_{HFsigname}_oneprobe.mat'    
    
# Load mat file with scatter plot data - for a description of the signals see * below
mat1 = scipy.io.loadmat(filen)

# Extract correct data
t_peaks = mat1[f't_p{toolLF}']
peaks_lf = np.squeeze(mat1[f'p{toolLF}'])
peaks_hf_matched = np.squeeze(mat1[f'mFXcorr_{toolLF}'])

# Also extract 'true' HF distribution
filen_true = f'{filepath}{testno}_FXpeaks_versus_Impulses.mat'
mat2 = scipy.io.loadmat(filen_true) 
# mat2.keys()
peaks_hf_true = np.squeeze(mat2['pFX'])
     
# # Check that true & matched HF peaks are not identical
# v_scr,p_scr = calc_distribution(np.squeeze(peaks_hf_matched))
# v_true,p_true = calc_distribution(np.squeeze(peaks_hf_true))
# plt.figure()
# plt.plot(v_true,p_true,'ob')
# plt.plot(v_scr,p_scr,'xr')
# plt.yscale('log')

# Check total number of HF events in the available duration
print(f'Total number of HF events in available duration: {len(peaks_hf_true)}')
      
# Run Adaptive Screening
debugplotswitch = 0
adaptive_screening(peaks_lf,peaks_hf_matched,peaks_hf_true,Tdata,acqHF,acqMF,maxitHF,maxitMF,prange,init_sampopt,init_plevels,dur,ne_exp,filefolder,figfolder,casename,varsymb,varunit,debugplotswitch)

# # Alternative
# adaptive_screening_gpfit(peaks_lf,peaks_hf_matched,peaks_hf_true,Tdata,acqGPfit,maxitGPfit,prange,init_sampopt,init_plevels,dur,ne_exp,filefolder,figfolder,casename,varsymb,varunit,debugplotswitch)



# # Close all figures
# plt.close('all')